<?php

class RouterController extends Controller
{

    protected $controller;
    protected $slugs = array(
        // "URL alias" => "Module/controller:akce?promena(podstranka)&promena &promena
        "account/profil" => "CoreModule/User?pageAction",
        "account/prihlaseni" => "CoreModule/Authentication:login",
        "account/odhlaseni" => "CoreModule/Authentication:logout",
        "projects/seznam" => "ProjectModule/ProjectList?id",
        "projects/novy" => "ProjectModule/Project:create"
    );

    public function action($parameters)
    {
        $parsedURL = $this->parseURL($parameters[0]); // vrátí všechno za lomítkem a před otazníkem | explode by "/"
        $queries = $this->getQueriesFromURL($parameters[0]); //e.g. id=10&q=ahoj
        $slug = $parsedURL[0]."/".$parsedURL[1];
        //echo $slug;

        //Je-li prázdnej URL alias (slug), tak redirect na úvod
        if ($slug == "/") {
                $this->redirect("account/prihlaseni");
        }

        //Vytvoření názvu akce a proměnných z URL aliasu
        if ($this->slugs[$slug]) {

            $explodeModule = explode("/",$this->slugs[$slug]);
            $explodeControllerAndQuery = explode("?", $explodeModule[1]); // (konroler:akce) a (promenna&promenna)
            $explodedControllerAndAction = explode(":", $explodeControllerAndQuery[0]);
            $explodedQueries = explode('&', $explodeControllerAndQuery[1]);

            $moduleName = $explodeModule[0];
            $controllerName = $explodedControllerAndAction[0];
            $action = $explodedControllerAndAction[1];

            if(strlen($action) == 0){
                $action = "default";
            }
            $actionParams = array("slug" => $slug, "action" => $action);

            unset($parsedURL[0]);

            if(count($explodedQueries) < count($parsedURL)-1 || (count($explodedQueries) == count($parsedURL)-2 && strlen($explodedQueries[0]) == 0)){
                $this->redirect("stranka-nenalezena");
            }

            if ($explodedQueries != NULL) {
                $i = 2;
                foreach ($explodedQueries as $query) {
                    $actionParams[$query] = $parsedURL[$i];
                    $i++;
                }
            }
        } else {
            $this->redirect("stranka-nenalezena");
        }

        //Vytvoření názvu classy kontroleru
        $controllerClass = $this->createMVCNameFromString($controllerName) . 'Controller';
        $modelClass = $this->createMVCNameFromString($controllerName) . 'Model';
        $model = new $modelClass($this->dataManager);

        //Kontrola existence kontroleru, jinak stranka nenalezena
        $this->controller = new $controllerClass($model);
        $this->controller->dataManager = $this->dataManager;

        $this->controller->module = $moduleName;

        //Volání handle metod v kontrolerech podle parametru v url "do" a vložení ostatních parametrů do handle metody; př. www.strnka.cz/podstranka.php?do=add&param1=hodnota1&param2=hodnota2
        if ($queries != NULL && strlen($queries["do"])) {
            $fnName = "handle" . ucfirst($queries["do"]);
            unset($queries["do"]);
            $this->controller->$fnName($queries);
        }

        //Volání action metod v kontrolerech a vložení parametrů do action metody; př. www.stranka.cz/podstranka/promena1/promena2 ..... v action metodě pak $parameters['promena']
        if ($action != NULL && $action != "default") {
            $fnName = "action" . ucfirst($action);
            if (method_exists($this->controller, $fnName)) {
                $this->controller->$fnName($this->dataManager,$actionParams);
            } else {
                $this->redirect("stranka-nenalezena");
            }
        } else {
            $this->controller->action($actionParams);
        }

        $this->controller->setController($this->controller);

        //Zavolá všechny metody controlleru, který začínají createComponent
        $controllMethods = get_class_methods($this->controller);
        if($controllMethods != NULL) {
            foreach ($controllMethods as $method) {
                if (preg_match("/^createComponent/", $method)) {
                    $this->controller->$method($actionParams);
                }
            }
        }

        $this->data['title'] = $this->controller->header['title'];
        $this->data['description'] = $this->controller->header['description'];
        $this->data['keywords'] = $this->controller->header['keywords'];
        $this->data['router'] = $this;
        $this->data['routedController'] = $this->controller;

        $this->view = $this->controller->mainLayout;
        $this->module = $this->controller->module;

    }

    private function parseURL($url)
    {
        $parsedURL = parse_url($url);
        $parsedURL["path"] = ltrim($parsedURL["path"], "/");
        $parsedURL["path"] = trim($parsedURL["path"]);
        return explode("/", $parsedURL["path"]);
        //vrátí všechno za lomítkem
    }

    private function getQueriesFromURL($url)
    {
        $parts = parse_url($url);
        parse_str($parts['query'], $query);
        return $query;
    }

    private function createMVCNameFromString($string)
    {
        return str_replace(" ", "", ucwords(str_replace("-", "", $string)));
    }
}