<?php
namespace Light;
use Light\ORM\DatabaseWorker;
use PDO;

class DBSessionHandler{
    private $savePath;
    private $sessionName;
    private $db = null;

    public function __construct() {
        DatabaseWorker::getInstance()->ConnectDatabase();
        $this->db = DatabaseWorker::getInstance()->getConnection();
        $this->db->setAttribute(PDO::ATTR_ERRMODE, PDO::ERRMODE_EXCEPTION);
        $this->db->setAttribute(PDO::ATTR_EMULATE_PREPARES, 0);

    }

    public function open($savePath, $sessionName) {
        $this->savePath = $savePath;
        $this->sessionName = $sessionName;
        return true;
    }

    public function close() {
        // your code if any
        $this->db = null;
        return true;
    }

    public function read($id)
    {
        $query = "SELECT data FROM `session` WHERE id=:id";
        $dbh = $this->db->prepare($query);
        $dbh->bindParam(":id", $id);

        if ($dbh->execute()) {
            $result = $dbh->fetchAll(PDO::FETCH_ASSOC);
            return $result[0]['data'];
        }else{
            return '';
        }
    }
    public function write($id, $data) {
        $access = time();

        $query = "REPLACE INTO `session` VALUES (:id, :access, :session_data);";
        $dbh = $this->db->prepare($query);

        $dbh->bindParam(":id", $id);
        $dbh->bindParam(":access", $access);
        $dbh->bindParam(":session_data", $data);

        try{
            $dbh->execute();
            return true;
        }catch (\PDOException $e){
            echo $e->getMessage();
        }
        return false;
    }

    public function destroy($id) {
        $query = "DELETE FROM `session` WHERE id = :id";
        $dbh = $this->db->prepare($query);
        $dbh->bindParam(":id", $id);

        if ($dbh->execute()) {
            return true;
        }
        return false;
    }

    public function gc($maxlifetime) {
        $access = time() - $maxlifetime;

        $query = "DELETE FROM session WHERE access < :old";
        $dbh = $this->db->prepare($query);
        $dbh->bindParam(":old", $access);

        if ($dbh->execute()) {
            return true;
        }
        return false;
    }
}
?>




